"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.base64 = void 0;
exports.createECDHSecret = createECDHSecret;
exports.ext = exports.default = void 0;
exports.privateKeyTweakAdd = privateKeyTweakAdd;
exports.privateKeyTweakMul = privateKeyTweakMul;
exports.privateKeyVerify = privateKeyVerify;
exports.pubKeyTweakAdd = pubKeyTweakAdd;
exports.pubKeyTweakMul = pubKeyTweakMul;
exports.publicKeyCreate = publicKeyCreate;
exports.sign = sign;
exports.utils = void 0;
exports.verify = verify;

var _reactNative = require("react-native");

var _utils = _interopRequireDefault(require("./utils"));

var _ext = _interopRequireDefault(require("./ext"));

var _secp256k1base = _interopRequireDefault(require("./secp256k1base64"));

var _errors = require("./errors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const LINKING_ERROR = `The package 'react-native-secp256k1-urnm' doesn't seem to be linked. Make sure: \n\n` + _reactNative.Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const Secp256k1Urnm = _reactNative.NativeModules.Secp256k1Urnm ? _reactNative.NativeModules.Secp256k1Urnm : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});

async function verify(sig, msg32, pubkey) {
  (0, _errors.isUint8Array)('signature', sig, 64);
  (0, _errors.isUint8Array)('message', msg32, 32);
  (0, _errors.isUint8Array)('public key', pubkey, [33, 65]);

  const bMsg32 = _utils.default.encodeBase64WithoutPadding(msg32);

  const bSig = _utils.default.encodeBase64WithoutPadding(sig);

  const bPub = _utils.default.encodeBase64WithoutPadding(pubkey);

  switch (await Secp256k1Urnm.verify(bMsg32, bSig, bPub)) {
    case 0:
      return true;

    case 3:
      return false;

    case 1:
      throw new Error(_errors.errors.SIG_PARSE);

    case 2:
      throw new Error(_errors.errors.PUBKEY_PARSE);
  }

  return false;
}

async function sign(msg32, seckey) {
  (0, _errors.isUint8Array)('message', msg32, 32);
  (0, _errors.isUint8Array)('private key', seckey, 32);

  const bMsg32 = _utils.default.encodeBase64WithoutPadding(msg32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  const bSignature = await Secp256k1Urnm.sign(bMsg32, bSeckey);

  switch (bSignature) {
    case 1:
      throw new Error(_errors.errors.SIGN);

    case 2:
      throw new Error(_errors.errors.IMPOSSIBLE_CASE);
  }

  return _utils.default.decodeBase64(bSignature);
}

async function privateKeyVerify(seckey) {
  (0, _errors.isUint8Array)('private key', seckey, 32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  return await Secp256k1Urnm.secKeyVerify(bSeckey);
}

async function publicKeyCreate(seckey, compressed) {
  (0, _errors.isUint8Array)('private key', seckey, 32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  const bPub = await Secp256k1Urnm.computePubkey(bSeckey, !!compressed);
  return _utils.default.decodeBase64(bPub);
}

async function createECDHSecret(seckey, pubkey) {
  (0, _errors.isUint8Array)('public key', pubkey, [33, 65]);
  (0, _errors.isUint8Array)('private key', seckey, 32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  const bPubkey = _utils.default.encodeBase64WithoutPadding(pubkey);

  const bSecret = await Secp256k1Urnm.createECDHSecret(bSeckey, bPubkey);
  return _utils.default.decodeBase64(bSecret);
}

async function privateKeyTweakMul(seckey, tweak) {
  (0, _errors.isUint8Array)('private key', seckey, 32);
  (0, _errors.isUint8Array)('tweak', tweak, 32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  const bTweak = _utils.default.encodeBase64WithoutPadding(tweak);

  const bResult = await Secp256k1Urnm.privKeyTweakMul(bSeckey, bTweak);
  return _utils.default.decodeBase64(bResult);
}

async function privateKeyTweakAdd(seckey, tweak) {
  (0, _errors.isUint8Array)('private key', seckey, 32);
  (0, _errors.isUint8Array)('tweak', tweak, 32);

  const bSeckey = _utils.default.encodeBase64WithoutPadding(seckey);

  const bTweak = _utils.default.encodeBase64WithoutPadding(tweak);

  const bResult = await Secp256k1Urnm.privKeyTweakAdd(bSeckey, bTweak);
  return _utils.default.decodeBase64(bResult);
}

async function pubKeyTweakAdd(pubkey, tweak) {
  (0, _errors.isUint8Array)('public key', pubkey, [33, 65]);
  (0, _errors.isUint8Array)('tweak', tweak, 32);

  const bPubkey = _utils.default.encodeBase64WithoutPadding(pubkey);

  const bTweak = _utils.default.encodeBase64WithoutPadding(tweak);

  const bResult = await Secp256k1Urnm.pubKeyTweakAdd(bPubkey, bTweak);
  return _utils.default.decodeBase64(bResult);
}

async function pubKeyTweakMul(pubkey, tweak) {
  (0, _errors.isUint8Array)('public key', pubkey, [33, 65]);
  (0, _errors.isUint8Array)('tweak', tweak, 32);

  const bPubkey = _utils.default.encodeBase64WithoutPadding(pubkey);

  const bTweak = _utils.default.encodeBase64WithoutPadding(tweak);

  const bResult = await Secp256k1Urnm.pubKeyTweakMul(bPubkey, bTweak);
  return _utils.default.decodeBase64(bResult);
}

const ext = _ext.default;
exports.ext = ext;
const utils = _utils.default;
exports.utils = utils;
const base64 = _secp256k1base.default;
exports.base64 = base64;
var _default = {
  verify,
  sign,
  privateKeyVerify,
  publicKeyCreate,
  privateKeyTweakAdd,
  privateKeyTweakMul,
  pubKeyTweakMul,
  pubKeyTweakAdd,
  createECDHSecret,
  ext,
  utils,
  base64
};
exports.default = _default;
//# sourceMappingURL=index.js.map